#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

#include "heap.h"
#include "trace.h"
#include "sched.h"

pstate **alloc_timeline(size_t timesteps, size_t nb_processes) {
    pstate** ptl  = malloc(nb_processes * sizeof(pstate *)); // processes x timesteps matrix
    for (size_t p=0; p<nb_processes; p++) {
        ptl[p] = malloc(timesteps*sizeof(pstate));
        for (size_t t=0; t<timesteps; t++ )
            ptl[p][t] = inactive;
    }
    return ptl;
}

void record_timeline(size_t timestep, pstate **timeline, heap *run, heap *pend) {
    for (size_t i=0; i< heap_size(run); i++) {
        heap_item *hi = heap_get_item(run,i);
        size_t pid = heap_item_get_value(hi);
        timeline[pid][timestep] = running;
    }
    for (size_t i=0; i< heap_size(pend); i++) {
        heap_item *hi = heap_get_item(pend,i);
        size_t pid = heap_item_get_value(hi);
        timeline[pid][timestep]= pending;
    }
}

void print_timeline(size_t timesteps, size_t nb_procs, pstate** ptl) {
    printf("\n[===Results===]\n");
    for (size_t p=0; p<nb_procs; p++) {
        printf("%zu\t", p);
        for (size_t t=0; t<timesteps; t++ )
            switch (ptl[p][t]) {
                case inactive: printf("_"); break;
                case running: printf("R"); break;
                case pending: printf("."); break;
            }
        printf("\n");
    }
}